@@ -424,9 +424,9 @@ remove_dir (GVfsBackendOnedrive *self,
 {
   GHashTableIter iter;
   MsgDriveItem *item;
-  g_autofree char *parent_id = NULL;
+  const char *parent_id = NULL;
 
-  parent_id = g_strdup (msg_drive_item_get_id (parent));
+  parent_id = msg_drive_item_get_id (parent);
 
   g_hash_table_remove (self->dir_timestamps, parent_id);
 
@@ -491,7 +491,7 @@ rebuild_shared_with_me_dir (GVfsBackendOnedrive  *self,
 
   timestamp = g_new (gint64, 1);
   *timestamp = g_get_real_time ();
-  g_hash_table_insert (self->dir_timestamps, SHARED_WITH_ME_ID, timestamp);
+  g_hash_table_insert (self->dir_timestamps, g_strdup (SHARED_WITH_ME_ID), timestamp);
 
   for (GList *l = items; l != NULL; l = l->next)
     {
@@ -692,6 +692,8 @@ build_file_info (GVfsBackendOnedrive    *self,
   GFileType file_type;
   const char *id;
   const char *name;
+  const char *user;
+  const char *etag;
   gboolean is_folder = FALSE;
   gboolean is_root = FALSE;
   gboolean is_home = (item == self->home);
@@ -778,10 +780,13 @@ build_file_info (GVfsBackendOnedrive    *self,
   g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_CREATED, msg_drive_item_get_created (item));
   g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED, msg_drive_item_get_modified (item));
 
-  g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_OWNER_USER_REAL, msg_drive_item_get_user (item));
-  g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_OWNER_USER, msg_drive_item_get_user (item));
+  user = msg_drive_item_get_user (item);
+  if (user)
+    g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_OWNER_USER_REAL, user);
 
-  g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_ETAG_VALUE, msg_drive_item_get_etag (item));
+  etag = msg_drive_item_get_etag (item);
+  if (etag)
+    g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_ETAG_VALUE, etag);
 
   if (!is_folder)
     {
@@ -1462,7 +1467,8 @@ g_vfs_backend_onedrive_seek_on_read (GVfsBackend       *_self,
 {
   GCancellable *cancellable = G_VFS_JOB (job)->cancellable;
   GError *error = NULL;
-  GInputStream *stream = G_INPUT_STREAM (handle);
+  ReadHandle *rh = (ReadHandle *)handle;
+  GInputStream *stream = rh->stream;
   goffset cur_offset;
 
   g_debug ("+ seek_on_read: %p\n", handle);
@@ -1491,6 +1497,7 @@ close_read_cb (GObject      *source_object,
   GError *error = NULL;
   GInputStream *stream = G_INPUT_STREAM (source_object);
   GVfsJobCloseRead *job = G_VFS_JOB_CLOSE_READ (user_data);
+  ReadHandle *rh = job->handle;
 
   g_input_stream_close_finish (stream, res, &error);
   if (error != NULL)
@@ -1502,7 +1509,7 @@ close_read_cb (GObject      *source_object,
   g_vfs_job_succeeded (G_VFS_JOB (job));
 
 out:
-  g_object_unref (stream);
+  read_handle_free (rh);
   g_debug ("- close_read\n");
 }
 
@@ -1517,7 +1524,6 @@ g_vfs_backend_onedrive_try_close_read (GVfsBackend       *backend,
   g_debug ("+ close_read: %p\n", handle);
 
   g_input_stream_close_async (rh->stream, G_PRIORITY_DEFAULT, cancellable, close_read_cb, job);
-  read_handle_free (rh);
   return TRUE;
 }
 
