@@ -141,6 +141,13 @@ try (GVfsJob *job)
   GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (op_job->backend);
   gboolean res;
 
+  if (g_vfs_backend_get_readonly_lockdown (op_job->backend))
+    {
+      g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                        _("Filesystem is read-only"));
+      return TRUE;
+    }
+
   if (class->try_move == NULL)
     return FALSE;
   
